/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_TARGETSINFO_H
#define DPC_IMPL_TYPE_TARGETSINFO_H

#include "impl_type_uint64.h"
#include "impl_type_uint16.h"
#include "dpc/impl_type_targetinfo.h"
#include "dpc/impl_type_lanemarkerinfo.h"
#include "dpc/impl_type_roadedgeinfo.h"

namespace dpc {
struct TargetsInfo {
    ::UInt64 timestamp{ 0U };
    ::UInt16 cyclecounter{ 0U };
    ::dpc::TargetInfo flniv;
    ::dpc::TargetInfo flniiv;
    ::dpc::TargetInfo flnriv;
    ::dpc::TargetInfo iv;
    ::dpc::TargetInfo slniv;
    ::dpc::TargetInfo slniiv;
    ::dpc::TargetInfo slnriv;
    ::dpc::TargetInfo iiv;
    ::dpc::TargetInfo riv;
    ::dpc::LaneMarkerInfo left_lane_marker;
    ::dpc::LaneMarkerInfo right_lane_marker;
    ::dpc::RoadEdgeInfo left_road_edge;
    ::dpc::RoadEdgeInfo right_road_edge;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(timestamp);
        fun(cyclecounter);
        fun(flniv);
        fun(flniiv);
        fun(flnriv);
        fun(iv);
        fun(slniv);
        fun(slniiv);
        fun(slnriv);
        fun(iiv);
        fun(riv);
        fun(left_lane_marker);
        fun(right_lane_marker);
        fun(left_road_edge);
        fun(right_road_edge);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(timestamp);
        fun(cyclecounter);
        fun(flniv);
        fun(flniiv);
        fun(flnriv);
        fun(iv);
        fun(slniv);
        fun(slniiv);
        fun(slnriv);
        fun(iiv);
        fun(riv);
        fun(left_lane_marker);
        fun(right_lane_marker);
        fun(left_road_edge);
        fun(right_road_edge);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("timestamp", timestamp);
        fun("cyclecounter", cyclecounter);
        fun("flniv", flniv);
        fun("flniiv", flniiv);
        fun("flnriv", flnriv);
        fun("iv", iv);
        fun("slniv", slniv);
        fun("slniiv", slniiv);
        fun("slnriv", slnriv);
        fun("iiv", iiv);
        fun("riv", riv);
        fun("left_lane_marker", left_lane_marker);
        fun("right_lane_marker", right_lane_marker);
        fun("left_road_edge", left_road_edge);
        fun("right_road_edge", right_road_edge);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("timestamp", timestamp);
        fun("cyclecounter", cyclecounter);
        fun("flniv", flniv);
        fun("flniiv", flniiv);
        fun("flnriv", flnriv);
        fun("iv", iv);
        fun("slniv", slniv);
        fun("slniiv", slniiv);
        fun("slnriv", slnriv);
        fun("iiv", iiv);
        fun("riv", riv);
        fun("left_lane_marker", left_lane_marker);
        fun("right_lane_marker", right_lane_marker);
        fun("left_road_edge", left_road_edge);
        fun("right_road_edge", right_road_edge);
    }

    friend bool operator==(const ::dpc::TargetsInfo& lhs, const ::dpc::TargetsInfo& rhs) noexcept
    {
        return (lhs.timestamp == rhs.timestamp) && (lhs.cyclecounter == rhs.cyclecounter) && (lhs.flniv == rhs.flniv) && (lhs.flniiv == rhs.flniiv) && (lhs.flnriv == rhs.flnriv) && (lhs.iv == rhs.iv) && (lhs.slniv == rhs.slniv) && (lhs.slniiv == rhs.slniiv) && (lhs.slnriv == rhs.slnriv) && (lhs.iiv == rhs.iiv) && (lhs.riv == rhs.riv) && (lhs.left_lane_marker == rhs.left_lane_marker) && (lhs.right_lane_marker == rhs.right_lane_marker) && (lhs.left_road_edge == rhs.left_road_edge) && (lhs.right_road_edge == rhs.right_road_edge);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_TARGETSINFO_H
